/* wptGPGMEWrapper.cpp - Object oriented wrapper for GPGME
 *	Copyright (C) 2006 Timo Schulz
 *
 * This file is part of WinPT.
 *
 * WinPT is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *  
 * WinPT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with WinPT; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <windows.h>
#include <stdio.h>
#include <string.h>

#include "gpgme.h"
#include "wptGPG.h"
#include "wptGPGME.h"
#include "wptW32API.h"


/* Constructor to build gpgme context. */
GPGME::GPGME ()
{
    gpgme_new (&ctx);
}

/* Destructor to release gpgme context. */
GPGME::~GPGME ()
{
    if (ctx != NULL)
	gpgme_release (ctx);
}

/* Enable or disable the feature to add the 
    WinPT version to armored output. */
void
GPGME::setChangeVersion (bool val)
{
    chg_ver = val;
}

/* Enable or disable armor. */
void 
GPGME::setArmor (bool val)
{
    gpgme_set_armor (ctx, val? 1 : 0);
}

bool
GPGME::getArmor (void)
{
    return gpgme_get_armor (ctx)? true: false;
}

/* Export key pattern @patt to file @outfile.
   Return value: 0 on success. */
gpgme_error_t
GPGME::exportToFile (const char *patt, const char *outfile)
{
    gpgme_data_t dat;
    gpgme_error_t err;

    err = gpgme_data_new (&dat);
    if (err)
	return err;
    op_begin ();
    err = gpgme_op_export (ctx, patt, 0, dat);
    op_end ();
    if (err) {
	gpgme_data_release (dat);
	return err;
    }
    err = gpg_data_release_and_set_file (dat, outfile);
    if (err)
	gpgme_data_release (dat);
    return err;
}


/* Export key pattern @patt to the clipboard.
   Return value: 0 on success. */
gpgme_error_t
GPGME::exportToClipboard (const char *patt)
{
    gpgme_data_t dat;
    gpgme_error_t err;

    err = gpgme_data_new (&dat);
    if (err)
	return err;
    op_begin ();
    err = gpgme_op_export (ctx, patt, 0, dat);
    op_end ();
    if (err) {
	gpgme_data_release (dat);
	return err;
    }
    gpg_data_release_and_set_clipboard (dat, chg_ver? 1 : 0);
    return err;
}


/* Export key pattern @patt to the buffer @outdata. 
   Return value: 0 on success. */
gpgme_error_t
GPGME::exportToBuffer (const char *patt, char **outdata)
{
    gpgme_error_t err;
    gpgme_data_t dat;
    size_t n;
    char *p;

    err = gpgme_data_new (&dat);
    if (err)
	return err;
    op_begin ();
    err = gpgme_op_export (ctx, patt, 0, dat);
    op_end ();
    if (err) {
	gpgme_data_release (dat);
	return err;
    }
    p = gpgme_data_release_and_get_mem (dat, &n);

    *outdata = new char[n+1];
    memcpy (*outdata, p, n);
    (*outdata)[n] = 0;
    gpgme_free (p);

    return err;
}
